from .exceptions import *
from .operations import *
import collections

class Domnibus(DomnibusOperationMixin):
    FUNC_PREFIX = '_get_value_for_'

    def __init__(self, domain):
        self.domain = domain
        self._data = {}

    def __getitem__(self, method):
        return self._get_and_save_data(method)

    def __getattr__(self, method):
        if method in self.allowed_methods():
            attr = self[method]
        else:
            attr = super(Domnibus, self).__getattr__(method)
        return attr

    @classmethod
    def allowed_methods(cls):
        allowed_methods = set()

        for func in dir(cls):
            if func.startswith(cls.FUNC_PREFIX) and isinstance(getattr(cls, func), collections.Callable):
                func_name = func.lstrip(cls.FUNC_PREFIX)
                allowed_methods.add(func_name)

        return allowed_methods

    def _get_and_save_data(self, key):
        if key in self.allowed_methods():

            if key not in self._data:
                value, store = getattr(self, '{}{}'.format(self.FUNC_PREFIX, key))()
                if store:
                    self._data[key] = value
            else:
                value = self._data[key]

            return value

        else:
            raise DomnibusMethodError("Method '{}' not implemented".format(key))




class DomnibusOperationMixin(object):

    def _get_value_for_whois(self):
        import whois
        return (whois.whois(self.domain), True)

    def _get_value_for_ssl(self):
        import ssl, socket

        ctx = ssl.create_default_context()
        s = ctx.wrap_socket(socket.socket(), server_hostname=self.domain)
        s.connect((self.domain, 443))
        return (s.getpeercert(), True)

    def _get_value_for_dns(self):
        import dns.resolver
        from dns.resolver import NoAnswer
        from collections import OrderedDict

        values = OrderedDict()
        for tp in ["A", "AAAA", "MX", "NS", "SOA", "TXT"]:
            try:
                values[tp] = [entry.to_text()
                              for entry in list(dns.resolver.query(self.domain, tp))]
            except NoAnswer:
                values[tp] = []
        return (values, True)
